<?php

namespace Drupal\contextual_reports;

use Drupal\contextual_reports\Entity\ContextualReportEntity;

class ContextualReportsUtilities {

  /*
   ****************************************************************************
   * Report Entities
   * Report entities are snapshots of report data, so that reports may be
   * archived and re-loaded at a future date. This allows for easy retrieval
   * and comparison of archived report data.
   ****************************************************************************
   */

  /**
   * @param $context_plugin
   * @param $data_plugin
   * @param $report_plugin
   * @param array $params
   * @param array $entity_data
   * @return int
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public static function createReportEntity($context_plugin, $data_plugin, $report_plugin, array $params = [], array $entity_data = []) {
    $data = ContextualReportsUtilities::generateReportData($context_plugin, $params);
    $entity_data['context_plugin'] = $context_plugin;
    $entity_data['data_plugin'] = $data_plugin;
    $entity_data['report_plugin'] = $report_plugin;
    $entity_data['params'] = $params;
    $entity_data['data'] = $data;
    $entity_data['tyoe'] = isset($entity_data['type']) ? $entity_data['type'] : 'default';
    $report = ContextualReportEntity::create($entity_data)->save();
    return $report;
  }

  /*
   ****************************************************************************
   * Reports
   ****************************************************************************
   */

  /**
   * @param $context_plugin
   * @param array $params
   * @return array
   */
  public static function getEntities($context_plugin, array $params = []) {
    $plugin_manager = \Drupal::service('plugin.manager.contextual_report_context');
    $plugin_definitions = $plugin_manager->getDefinitions();
    if (isset($plugin_definitions[$context_plugin]['class'])) {
      return $plugin_definitions[$context_plugin]['class']::getEntities($params);
    }
    // We have an invalid context plugin.
    return [];
  }

  /**
   * @param $context_plugin
   * @param $data_plugin
   * @param array $params
   * @return array
   */
  public static function generateReportData($context_plugin, $data_plugin, array $params = []) {
    $entity_ids = ContextualReportsUtilities::getEntities($context_plugin, $params);
    $plugin_manager = \Drupal::service('plugin.manager.contextual_report_data');
    $plugin_definitions = $plugin_manager->getDefinitions();
    if (isset($plugin_definitions[$data_plugin]['class'])) {
      return $plugin_definitions[$data_plugin]['class']::generateReportData($entity_ids, $params);
    }
    // We have an invalid report plugin.
    return [];
  }

  /**
   * @param $context_plugin
   * @param $data_plugin
   * @param $report_plugin
   * @param array $params
   * @return array
   */
  public static function generateReport($context_plugin, $data_plugin, $report_plugin, array $params = []) {
    $report_data = ContextualReportsUtilities::generateReportData($context_plugin, $data_plugin, $params);
    if (!empty($report_data)) {
      $plugin_manager = \Drupal::service('plugin.manager.contextual_report');
      $plugin_definitions = $plugin_manager->getDefinitions();
      if (isset($plugin_definitions[$report_plugin]['class'])) {
        return $plugin_definitions[$report_plugin]['class']::generateReport($report_data, $params);
      }
    }
    // We have no data, or we have an invalid report plugin.
    return [];
  }



}